@@ -3,9 +3,9 @@
 ## Author: Brice Ozenne
 ## Created: mar  5 2021 (21:28) 
 ## Version: 
-## Last-Updated: nov 13 2021 (16:35) 
+## Last-Updated: Feb 13 2022 (23:12) 
 ##           By: Brice Ozenne
-##     Update #: 471
+##     Update #: 483
 ##----------------------------------------------------------------------
 ## 
 ### Commentary: 
@@ -25,7 +25,7 @@
 ##' or only for coefficients relative to the mean (\code{"mean"} or \code{"fixed"}),
 ##' or only for coefficients relative to the variance structure (\code{"variance"}),
 ##' or only for coefficients relative to the correlation structure (\code{"correlation"}).
-##' @param robust [logical] Should robust standard error (aka sandwich estimator) be output instead of the model-based standard errors. Not feasible for variance or correlation coefficients estimated by REML.
+##' @param robust [logical] Should robust standard errors (aka sandwich estimator) be output instead of the model-based standard errors. Not feasible for variance or correlation coefficients estimated by REML.
 ##' @param df [logical] Should degree of freedom, computed using Satterthwaite approximation, for the model parameters be output.
 ##' @param data [data.frame] dataset relative to which the information should be computed. Only relevant if differs from the dataset used to fit the model.
 ##' @param p [numeric vector] value of the model coefficients at which to evaluate the information. Only relevant if differs from the fitted values.
@@ -107,18 +107,7 @@ vcov.lmm <- function(object, effects = "mean", robust = FALSE, df = FALSE, strat
             test.precompute <- !is.null(object$design$precompute.XX)
          
             if(!is.null(data)){
-                ff.allvars <- c(all.vars(object$formula$mean), all.vars(object$formula$var))
-                if(any(ff.allvars %in% names(data) == FALSE)){
-                    stop("Incorrect argument \'data\': missing variable(s) \"",paste(ff.allvars[ff.allvars %in% names(data) == FALSE], collapse = "\" \""),"\".\n")
-                }
-
-                design <- .model.matrix.lmm(formula.mean = object$formula$mean.design,
-                                            structure = object$design$vcov,
-                                            data = data,
-                                            U.strata = object$strata$levels,
-                                            U.time = object$time$levels,
-                                            stratify.mean = object$opt$name=="gls",
-                                            precompute.moments = test.precompute)
+                design <- stats::model.matrix(object, data = data, effects = "all", simplifies = FALSE)
             }else{
                 design <- object$design
             }
